/* $Id: checksum.c,v 1.2 2002/08/04 14:03:21 jirka Exp $ */

/*
 * netrw tools
 * Copyright (C) 2002 Jiri Denemark
 *
 * This file is part of netrw.
 *
 * netrw is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define NEED_CHECKSUMS

#include "md5.h"
#include "common.h"
#include "proto_types.h"
#include "proto.h"
#include "net.h"
#include "checksum.h"
#include "checksum_proto.h"

#undef NEED_CHECKSUMS

static int type = CHECKSUM_NONE;
static md5_t md5;
static char checksum[MD5_SIZE * 2 + 1];
static char remote_sum[MD5_SIZE * 2 + 1];


void checksum_init(const struct proto_params *params)
{
    type = params->checksum;

    memset(checksum, '\0', sizeof(checksum));
    memset(remote_sum, '\0', sizeof(remote_sum));

    md5_init(&md5);
}

void checksum_process(const void *buffer, unsigned int len)
{
    md5_process(&md5, buffer, len);
}

char *checksum_finish(void)
{
    char sig[MD5_SIZE];

    md5_finish(&md5, (void *) sig);
    md5_sig_to_string((void *) sig, checksum, sizeof(checksum));

    return checksum;
}

char *checksum_exchange(void)
{
    char header[100];
    char *line;
    int len;

    len = snprintf(header, 100, "%s(%s)", PAR_STR_CHECKSUM, CHECKSUMS[type]);

    if (control_writef(SM_IMMEDIATELY, "%s: %s",
                       header, checksum) == FN_FAILURE) {
        print(ERROR, "cannot send %s checksum\n", CHECKSUMS[type]);
        return NULL;
    }

    if ((line = control_read()) == NULL) {
        print(ERROR, "cannot read %s checksum\n", CHECKSUMS[type]);
        return NULL;
    }

    if (line_header(&line, header) == FN_FAILURE
        || *line != ' '
        || strlen(++line) != MD5_SIZE * 2) {
        print(ERROR, "invalid checksum specification: ``%s''\n", line);
        return NULL;
    }

    strncpy(remote_sum, line, MD5_SIZE * 2);
    remote_sum[MD5_SIZE * 2] = '\0';

    return remote_sum;
}

int checksum_validate(void)
{
    if (memcmp((void *) checksum, (void *) remote_sum, MD5_SIZE * 2) == 0)
        return FN_SUCCESS;
    else
        return FN_FAILURE;
}


void checksum_proto_param_options(char *options,
                                  struct proto_params *req_params,
                                  struct proto_params *params)
{
    int i;
    int opt = -1;
    char *option;

    /* not requested by any party, ignore it */
    if (req_params->checksum == PAR_NOT_REQ
        && params->checksum == CHECKSUM_NONE)
        return;

    /* redefined, ignore it */
    if (req_params->checksum == PAR_DEFINED
        || req_params->checksum == PAR_REDEFINED) {
        req_params->checksum = PAR_REDEFINED;
        print(WARNING, "checksum parameter redefined; ignored\n");
        return;
    }

    while ((option = get_word(&options)) != NULL) {
        for (i = 0; i < CHECKSUMS_CNT; i++) {
            if (strcmp(option, CHECKSUMS[i]) == 0)
                break;
        }
        if (i == CHECKSUMS_CNT)
            continue;

        if (opt == -1 || params->checksum == i)
            opt = i;
    }

    if (opt > -1) {
        print(VERBOSE, "checksum algorithm: %s\n", CHECKSUMS[opt]);
        params->checksum = opt;
    }
    else {
        print(WARNING, "your party doesn't offer any suitable "
              "checksum algorithm; checksum disabled\n");
        params->checksum = CHECKSUM_NONE;
    }

    req_params->checksum = PAR_DEFINED;
}

void checksum_proto_check(struct proto_params *req_params,
                          struct proto_params *params)
{
    if (req_params->checksum == PAR_REQUESTED) {
        print(WARNING, "your party requires checksum but doesn't support "
              "any checksum algorithm; checksum disabled\n");
        params->checksum = CHECKSUM_NONE;
    }
    else if (req_params->checksum == PAR_NOT_REQ
             && params->checksum != CHECKSUM_NONE) {
        print(WARNING, "your party doesn't know anything about checksum "
              "parameter you have requested; checksum disabled\n");
        params->checksum = CHECKSUM_NONE;
    }
}

int checksum_proto_accept_option(char *option,
                                 struct proto_params *req_params,
                                 struct proto_params *params)
{
    int i;

    /* redefined, ignore it */
    if (req_params->checksum == PAR_DEFINED
        || req_params->checksum == PAR_REDEFINED) {
        req_params->checksum = PAR_REDEFINED;
        print(WARNING, "checksum parameter redefined; ignored\n");
        return FN_SUCCESS;
    }

    if (option != NULL) {
        for (i = 0; i < CHECKSUMS_CNT; i++) {
            if (strcmp(option, CHECKSUMS[i]) == 0) {
                params->checksum = i;
                break;
            }
        }
    }

    if (params->checksum == CHECKSUM_NONE) {
        print(ERROR, "checksum algorithm negotiation failed\n");
        return FN_FAILURE;
    }
    else {
        print(VERBOSE, "checksum algorithm: %s\n",
              CHECKSUMS[params->checksum]);
    }

    return FN_SUCCESS;
}

